<?php

namespace bhr\Admin;

if(!defined('ABSPATH')) exit;

use bhr\Admin\Controller\SettingsController as SettingsController;
use bhr\Admin\Entity\MessageEntity;
use bhr\Admin\Model\AdminModel;
use bhr\Admin\Model\Helper;
use SALESmanago\Exception\Exception;

class Admin
{
    protected $SettingsController;
    protected $AdminModel;

    public function __construct()
    {
        try {
            load_plugin_textdomain('salesmanago', false, 'salesmanago/languages');
            add_action('admin_enqueue_scripts', array($this, 'registerAssets'));
        } catch (\Exception $e) {
            MessageEntity::getInstance()->addException(new Exception($e->getMessage(), 603));
        }

        try {
            $this->AdminModel = new AdminModel();
            $this->SettingsController = new SettingsController($this->AdminModel);
        } catch (\Exception $e) {
            MessageEntity::getInstance()->addException(new Exception($e->getMessage(), 600));
        }

        $this->AdminModel->getConfigurationFromDb();
        $this->AdminModel->getPlatformSettingsFromDb();

        if(isset($_REQUEST['page']) && strpos($_REQUEST['page'], 'salesmanago') !== false) {
            try {
                $this->SettingsController->route();
            } catch (\Exception $e) {
                MessageEntity::getInstance()->addException(new Exception($e->getMessage(), 604));
            }
        }
        try {
            $this->SettingsController->setUserLogged();
            if($this->AdminModel->getUserLogged()) {
                $this->SettingsController->refreshOwnerList();
            }
            $this->SettingsController->setAvailableTabs();
        } catch (\Exception $e) {
            MessageEntity::getInstance()->addException(new Exception($e->getMessage(), 500));
        }
        add_action('admin_menu', array($this, 'registerAdminDashboardPage'));
    }

    /**
     *
     */
    public function registerAdminDashboardPage()
    {
        try {
            $this->SettingsController->registerMenuPages();
        } catch (\Exception $e) {
            MessageEntity::getInstance()->addException(new Exception($e->getMessage(), 601));
        }
    }

    /**
     *
     */
    public function registerAssets()
    {
        try {
        wp_enqueue_style('salesmanago', plugin_dir_url( __FILE__ ) . 'View/css/salesmanago-admin.css', array(), SM_VERSION, 'all');
        wp_enqueue_script('salesmanago', plugin_dir_url( __FILE__ ) . 'View/js/salesmanago-admin.js', array(), SM_VERSION, true);
        } catch (\Exception $e) {
            MessageEntity::getInstance()->addException(new Exception($e->getMessage(), 603));
        }
    }
}
