<?php
namespace bhr\admin\Controller;

if(!defined('ABSPATH')) exit;

use bhr\Admin\Entity\MessageEntity;
use bhr\Admin\Model\AdminModel;
use bhr\Admin\View\SettingsRenderer;
use bhr\Admin\Controller\LoginController;

use SALESmanago\Exception\Exception;
use SALESmanago\Services\UserAccountService;

class SettingsController
{
    public $SettingsRenderer;
    private $AdminModel;
    protected $UserModel;


    public function __construct(AdminModel $AdminModel)
    {
        $this->AdminModel = $AdminModel;

        if (isset($_REQUEST['page'])) {
            $this->AdminModel->setPage($_REQUEST['page']);
        }

    }

    /**
     *
     */
    public function includeSettingsView()
    {
        $this->AdminModel->setInstalledPlugins();
        $this->SettingsRenderer = new SettingsRenderer($this->AdminModel);
        $this->SettingsRenderer->getSettingsView();
    }

    /**
     *
     */
    public function registerMenuPages()
    {
        //General menu position (the one with icon)
        add_menu_page(
            __('General settings of SALESmanago integration', 'salesmanago'),
            __('SALESmanago', 'salesmanago'),
            'manage_options',
            SALESMANAGO,
            array($this, 'includeSettingsView'),
            $this->AdminModel->getIconBase64(),
            55
        );

        //First submenu position (this will open as default)
        if ($this->AdminModel->getUserLogged()) {
            //If user logged - show Integration Settings
            add_submenu_page(
                'salesmanago',
                __('Manage SM integration settings', 'salesmanago'),
                __('Integration settings', 'salesmanago'),
                'manage_options',
                SALESMANAGO,
                array($this, 'includeSettingsView')
            );
        } else {
            //Otherwise - show login screen
            add_submenu_page(
                'salesmanago',
                __('Login to SALESmanago account', 'salesmanago'),
                __('Login', 'salesmanago'),
                'manage_options',
                SALESMANAGO,
                array($this, 'includeSettingsView')
            );
        }

        //Other tabs
        //Export
        if ($this->AdminModel->isTabAvailable("salesmanago-export")) {
            add_submenu_page(
                'salesmanago',
                __('Export contacts and events', 'salesmanago'),
                __('Export', 'salesmanago'),
                'manage_options',
                SALESMANAGO . '-export',
                array($this, 'includeSettingsView')
            );
        }
        //Plugins
        if ($this->AdminModel->isTabAvailable("salesmanago-plugins")) {
            add_submenu_page(
                'salesmanago',
                __('Manage integrations with plugins', 'salesmanago'),
                __('Plugins', 'salesmanago'),
                'manage_options',
                SALESMANAGO . '-plugins',
                array($this, 'includeSettingsView')
            );
        }
        //Plugins - Wordpress
        if ($this->AdminModel->isTabAvailable("salesmanago-plugin-wp")) {
            add_submenu_page(
                'salesmanago',
                __('Manage WordPress integration', 'salesmanago'),
                __('WordPress', 'salesmanago'),
                'manage_options',
                SALESMANAGO . '-plugin-wp',
                array($this, 'includeSettingsView')
            );
        }
        //Plugins - WooCommerce
        if ($this->AdminModel->isTabAvailable("salesmanago-plugin-wc")) {
            add_submenu_page(
                'salesmanago',
                __('Manage WooCommerce integration', 'salesmanago'),
                __('WooCommerce', 'salesmanago'),
                'manage_options',
                SALESMANAGO . '-plugin-wc',
                array($this, 'includeSettingsView')
            );
        }
        //Plugins - Contact Form 7
        if ($this->AdminModel->isTabAvailable("salesmanago-plugin-cf7")) {
            add_submenu_page(
                'salesmanago',
                __('Manage Contact Form 7 integration', 'salesmanago'),
                __('Contact Form 7', 'salesmanago'),
                'manage_options',
                SALESMANAGO . '-plugin-cf7',
                array($this, 'includeSettingsView')
            );
        }
        //Plugins - Gravity Forms
        if ($this->AdminModel->isTabAvailable("salesmanago-plugin-gf")) {
            add_submenu_page(
                'salesmanago',
                __('Manage Gravity Forms integration', 'salesmanago'),
                __('Gravity Forms', 'salesmanago'),
                'manage_options',
                SALESMANAGO . '-plugin-gf',
                array($this, 'includeSettingsView')
            );
        }

        add_submenu_page(
            'salesmanago',
            __('Manage integrations with plugins', 'salesmanago'),
            __('SALESmanago.com', 'salesmanago'),
            'manage_options',
            SALESMANAGO . '-go-to-app',
            array($this, 'includeSettingsView')
        );
    }

    /**
     *
     */
    public function setUserLogged()
    {
        try{
            if (!empty($this->AdminModel->getConfiguration()->isActive())
            && !empty($this->AdminModel->getConfiguration()->getToken())) {
                $this->AdminModel->setUserLogged(true);
            }
        } catch (\Exception $e) {
            MessageEntity::getInstance()->addException(new Exception($e->getMessage(), 500));
        }
    }

    /**
     * @throws Exception
     */
    public function route()
    {
        if(!empty($_REQUEST['action'])) {
            switch ($_REQUEST['action']) {
                case 'login':
                    $LoginController = new LoginController($this->AdminModel);
                    $LoginController->loginUser($_REQUEST);
                    break;
                case 'logout':
                    $LoginController = new LoginController($this->AdminModel);
                    $LoginController->logoutUser();
                    break;
                case 'save':
                    $this->AdminModel->parseSettingsFromRequest($_REQUEST);
                    $this->AdminModel->saveConfiguration();
                    $this->AdminModel->savePlatformSettings();
                    MessageEntity::getInstance()->addMessage('Settings have been saved.', 'success', 703);
            }
        }
        if(!empty($_REQUEST['message'])) {
            switch ($_REQUEST['message']) {
                case 'logout':
                    MessageEntity::getInstance()->addMessage('Logged out.',  'success', 702);
                    break;
                case 'logout-error':
                        MessageEntity::getInstance()->addException(new Exception(__('Error on logout'), 151));
                    break;
            }
        }
    }

    /**
     *
     */
    public function setAvailableTabs()
    {
        if ($this->AdminModel->getUserLogged()) {
            $this->AdminModel->setAvailableTabs(array(SALESMANAGO, SALESMANAGO . '-export', SALESMANAGO . '-plugins'));

            if ($this->AdminModel->getPlatformSettings()->isActive('wp')) {
                $this->AdminModel->appendAvailableTabs(SALESMANAGO . '-plugin-wp');
            }
            if ($this->AdminModel->getPlatformSettings()->isActive('wc')) {
                $this->AdminModel->appendAvailableTabs(SALESMANAGO . '-plugin-wc');
            }
            if ($this->AdminModel->getPlatformSettings()->isActive('cf7')) {
                $this->AdminModel->appendAvailableTabs(SALESMANAGO . '-plugin-cf7');
            }
            if ($this->AdminModel->getPlatformSettings()->isActive('gf')) {
                $this->AdminModel->appendAvailableTabs(SALESMANAGO . '-plugin-gf');
            }
        } else {
            $this->AdminModel->setAvailableTabs(array(SALESMANAGO));
        }

    }

    public function refreshOwnerList()
    {
        $UserAccountService = new UserAccountService($this->AdminModel->Configuration);
        try {
            $Response = $UserAccountService->getOwnersList();
            if(!$Response->getStatus()) {
                MessageEntity::getInstance()->addMessage("False response while getting owner list", 120);
            }
            $owners = $Response->getField('users');
            if(!empty($owners) && is_array($owners)) {
                $this->AdminModel->Configuration->setOwnersList($owners);
            }
        } catch (Exception $e) {
            MessageEntity::getInstance()->addException(new Exception($e->getMessage(), 120));
        }
    }

}