<?php

namespace bhr\Admin\Entity;

if(!defined('ABSPATH')) exit;

use bhr\Admin\Entity\Plugins\Wp as PluginWp;
use bhr\Admin\Entity\Plugins\Wc as PluginWc;
use bhr\Admin\Entity\Plugins\Cf7 as PluginCf7;
use bhr\Admin\Entity\Plugins\Gf as PluginGf;
use SALESmanago\Exception\Exception;

class PlatformSettings implements \JsonSerializable
{
    private static $instances = [];

    protected $PluginWp;
    protected $PluginWc;
    protected $PluginCf7;
    protected $PluginGf;

    protected $updatedAt;
    protected $disableMonitoringCode = false;
    protected $languageDetection     = 'platform';

    private function __construct()
    {
        $this->PluginWp  = new PluginWp();
        $this->PluginWc  = new PluginWc();
        $this->PluginCf7 = new PluginCf7();
        $this->PluginGf  = new PluginGf();
    }


    protected function __clone()
    {
    }

    /**
     * @throws Exception
     */
    public function __wakeup()
    {
        throw new Exception("Cannot unserialize a singleton.");
    }

    /**
     * @return mixed|static
     */
    public static function getInstance()
    {
        $cls = static::class;
        if (!isset(self::$instances[$cls])) {
            self::$instances[$cls] = new static();
        }

        return self::$instances[$cls];
    }

    /**
     * @return mixed
     */
    public function getUpdatedAt()
    {
        return $this->updatedAt;
    }

    /**
     * @param mixed $updatedAt
     */
    public function setUpdatedAt($updatedAt)
    {
        $this->updatedAt = $updatedAt;
    }

    /**
     * @return PluginWp
     */
    public function getPluginWp()
    {
        return $this->PluginWp;
    }


    /**
     * @param $PluginWp
     * @return $this
     */
    public function setPluginWp($PluginWp)
    {
        $this->PluginWp = $PluginWp;
        return $this;
    }

    /**
     * @return PluginWc
     */
    public function getPluginWc()
    {
        return $this->PluginWc;
    }


    /**
     * @param $PluginWc
     * @return $this
     */
    public function setPluginWc($PluginWc)
    {
        $this->PluginWc = $PluginWc;
        return $this;
    }

    /**
     * @return PluginCf7
     */
    public function getPluginCf7()
    {
        return $this->PluginCf7;
    }


    /**
     * @param $PluginCf7
     * @return $this
     */
    public function setPluginCf7($PluginCf7)
    {
        $this->PluginCf7 = $PluginCf7;
        return $this;
    }

    /**
     * @return PluginGf
     */
    public function getPluginGf()
    {
        return $this->PluginGf;
    }


    /**
     * @param $PluginGf
     * @return $this
     */
    public function setPluginGf($PluginGf)
    {
        $this->PluginGf = $PluginGf;
        return $this;
    }

    /**
     * @param $name
     * @return PluginCf7|PluginGf|PluginWc|PluginWp|null
     */
    public function getPluginByName($name)
    {
        if ($name == 'wp') {
            return $this->PluginWp;
        } elseif (($name == 'wc')) {
            return $this->PluginWc;
        } elseif (($name == 'cf7')) {
            return $this->PluginCf7;
        } elseif (($name == 'gf')) {
            return $this->PluginGf;
        }
        return null;
    }

    /**
     * @param $name
     * @return bool|null
     */
    public function isActive($name)
    {
        if ($name == 'wp') {
            return $this->PluginWp->isActive();
        } elseif (($name == 'wc')) {
            return $this->PluginWc->isActive();
        } elseif (($name == 'cf7')) {
            return $this->PluginCf7->isActive();
        } elseif (($name == 'gf')) {
            return $this->PluginGf->isActive();
        }
        return null;
    }


    /**
     * @return array|mixed
     */
    public function jsonSerialize()
    {
        $out = array();
        foreach (get_object_vars($this) as $key => $val) {
            if (is_object($val)) {
                $out[$key] = $val->toArray();
            } else {
                $out[$key] = $val;
            }
        }
        return $out;
    }

    /**
     * @return array
     */
    private function toArray()
    {
        $out = array();
        foreach (get_object_vars($this) as $key => $val) {
            if (is_object($val) && !is_array($val)) {
                $out[$key] = $val->toArray();
            } else {
                $out[$key] = $val;
            }
        }
        return $out;
    }

    /**
     * @return bool
     */
    public function isDisableMonitoringCode()
    {
        return $this->disableMonitoringCode;
    }

    /**
     * @param bool $disableMonitoringCode
     * @return PlatformSettings
     */
    public function setDisableMonitoringCode($disableMonitoringCode = false)
    {
        $this->disableMonitoringCode = $disableMonitoringCode;
        return $this;
    }

    /**
     * @return string
     */
    public function getLanguageDetection()
    {
        return $this->languageDetection;
    }

    /**
     * @param string $languageDetection
     * @return PlatformSettings
     */
    public function setLanguageDetection($languageDetection = 'platform')
    {
        if (!empty($languageDetection) && ($languageDetection === 'platform' || $languageDetection === 'browser')) {
            $this->languageDetection = $languageDetection;
        }
        return $this;
    }

}