<?php

namespace bhr\Admin\Entity\Plugins;

if(!defined('ABSPATH')) exit;

use bhr\Admin\Entity\AbstractEntity;

class OptInInput extends AbstractEntity
{
    protected $mode       = 'none';
    protected $label      = 'Subscribe to our newsletter';
    protected $mappedName = 'sm-optin';

    /**
     * @return string
     */
    public function getMode()
    {
        return $this->mode;
    }

    /**
     * @param string $mode
     */
    public function setMode($mode)
    {
        $this->mode = $mode;
        return $this;
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return $this->label;
    }

    /**
     * @param string $label
     */
    public function setLabel($label)
    {
        $this->label = $label;
        return $this;
    }

    /**
     * @return string
     */
    public function getMappedName()
    {
        return $this->mappedName;
    }

    /**
     * @param string $mappedName
     */
    public function setMappedName($mappedName)
    {
        $this->mappedName = $mappedName;
        return $this;
    }

    public function setOptInInput($optInInput)
    {
        if(is_array($optInInput)) {
            $this->mode       = isset($optInInput['mode']) ? $optInInput['mode'] : 'none';
            $this->label      = isset($optInInput['label']) ? $optInInput['label'] : 'Subscribe to our newsletter';
            $this->mappedName = isset($optInInput['name']) ? $optInInput['name'] : 'sm-optin';
        } else {
            $this->mode = isset($optInInput->mode) ? $optInInput->mode : 'none';
            $this->label = isset($optInInput->label) ? $optInInput->label : 'Subscribe to our newsletter';
            $this->mappedName = isset($optInInput->mappedName) ? $optInInput->mappedName : 'sm-optin';
        }
        if($this->mode === 'append' || $this->mode === 'appendEverywhere') {
            if (function_exists('icl_register_string')) {
                icl_register_string('salesmanago', 'OptInInputLabel', $this->mappedName);
            }
        }
        return $this;
    }

    public function setLegacyOptInInput($optInInput, $active = false)
    {
        $mode = 'none';
        if(!empty($optInInput->type)) {
            switch($optInInput->type) {
                case 'newsletter':
                    $mode = 'append';
                    break;
                case 'mapper':
                    $mode = 'map';
                    break;
                default:
                    $mode = 'none';
                    break;
            }
        }
        $label = isset($optInInput->newsletterContent->default) ? $optInInput->newsletterContent->default : 'Subscribe to our newsletter';
        $mappedName = isset($optInInput->mappedName) ? $optInInput->mappedName : 'sm-optin';

        if(!$active) {
            $mode = 'none';
        }
        $this->setOptInInput(array(
            'mode' => $mode,
            'label' => $label,
            'mappedName' => $mappedName
        ));
    }
}