<?php

namespace bhr\Admin\Model;

if(!defined('ABSPATH')) exit;

use bhr\Admin\Model\AdminModel;
use bhr\Admin\Model\Helper;
use SALESmanago\Exception\Exception;

use SALESmanago\Entity\Contact\Contact;
use SALESmanago\Entity\Contact\Options;
use SALESmanago\Entity\Contact\Properties;
use SALESmanago\Entity\Contact\Address;
use SALESmanago\Entity\Event\Event;

use SALESmanago\Model\Collections\ContactsCollection;
use SALESmanago\Model\Collections\EventsCollection;

use SALESmanago\Controller\ExportController as SMExportController;

class ExportModel
{
    const
        PACKAGE_SIZE = 300,

        PREPARING    = 'preparing',
        FAILED       = 'failed',
        IN_PROGRESS  = 'in_progress',
        LAST_CHECK   = 'last_check',
        DONE         = 'done',

        CONTACTS     = 'contacts',
        EVENTS       = 'events',
        PURCHASE     = 'PURCHASE';

    protected $db;
    protected $AdminModel;

    protected $exportType;
    protected $dateFrom;
    protected $dateTo;
    protected $tags;
    protected $lastExportedPackage = -1;  //No packages have been exported. 0 will be the first one.
    protected $started;
    protected $lastSuccess;
    protected $packageCount = 0;
    protected $status = 'unknown';
    protected $message = '';

    public function __construct()
    {
        $this->db = $GLOBALS['wpdb'];
    }


    /**
     * @param int $packageCount
     */
    public function setPackageCount($packageCount)
    {
        $this->packageCount = $packageCount;
    }

    /**
     * @return int
     */
    public function getLastExportedPackage()
    {
        return $this->lastExportedPackage;
    }

    /**
     * @param int $lastExportedPackage
     */
    public function setLastExportedPackage($lastExportedPackage)
    {
        $this->lastExportedPackage = $lastExportedPackage;
    }

    /**
     * @param mixed $exportType
     */
    public function setExportType($exportType)
    {
        $this->exportType = $exportType;
    }

    /**
     * @return int
     */
    public function getPackageCount()
    {
        return $this->packageCount;
    }

    /**
     * @param string $status
     */
    public function setStatus($status)
    {
        $this->status = $status;
    }

    /**
     * @param string $message
     */
    public function setMessage($message)
    {
        $this->message = $message;
    }

    /**
     *
     */
    public function parseArgs()
    {
        try {
            $data = json_decode(base64_decode($_REQUEST['data']));

            $this->dateFrom = empty($data->dateFrom)
                ? '2000-01-01'
                : $data->dateFrom;

            $this->dateTo = empty($data->dateTo)
                ? date('Y-m-d', time() + 86400)
                : date('Y-m-d', strtotime($data->dateTo) + 86400);

            $this->tags = empty($data->tags)
                ? array()
                : Helper::clearCSVInput($data->tags, false, true, true);

            $this->lastExportedPackage = isset($data->lastExportedPackage)
                ? intval($data->lastExportedPackage)
                : -1;

            $this->packageCount = empty($data->packageCount)
                ? 0
                : intval($data->packageCount);

            $this->started = empty($data->started)
                ? time()
                : intval($data->started);

            $this->lastSuccess = empty($data->lastSuccess)
                ? 0
                : intval($data->lastSuccess);
        } catch (\Exception $e) {
            $this->message = $e->getMessage();
            $this->status = self::FAILED;
            $this->buildResponse();
        }
    }

    /**
     *
     */
    public function buildResponse()
    {
        $response = array(
            'packageSize'         => self::PACKAGE_SIZE,
            'packageCount'        => $this->packageCount,
            'lastExportedPackage' => $this->lastExportedPackage,
            'started'             => $this->started,
            'lastSuccess'         => time(),
            'type'                => $this->exportType,
            'tags'                => $this->tags,
            'status'              => $this->status,
            'message'             => $this->message,
        );
        echo(json_encode($response));
        die();
    }



    /**
     * @param $collection
     * @return ContactsCollection|null
     */
    public function prepareContactsToExport($collection)
    {
        try {
            $ContactsCollection = new ContactsCollection();

            foreach ($collection as $customer) {
                if (empty($customer['email'])) {
                    continue;
                }

                $Contact = new Contact();
                $Options = new Options();
                $Address = new Address();
                $Properties = new Properties();
                $Contact->setOptions($Options);
                $Contact->setAddress($Address);
                $Contact->setProperties($Properties);

                /* Contact */
                $customer['name'] = trim(
                    (isset($customer['first_name']) ? $customer['first_name'] : '') .
                    ' ' .
                    (isset($customer['last_name']) ? $customer['last_name'] : '')
                );
                $Contact
                    ->setEmail(isset($customer['email']) ? $customer['email'] : null)
                    ->setName(isset($customer['name']) ? $customer['name'] : null)
                    ->setExternalId(isset($customer['user_id']) ? $customer['user_id'] : null)
                    ->setPhone(isset($customer['phone']) ? $customer['phone'] : null);

                /* Address */
                $customer['address'] = trim(
                    (isset($customer['address_1']) ? $customer['address_1'] : '') .
                    ' ' .
                    (isset($customer['address_2']) ? $customer['address_2'] : '')
                );
                $Address
                    ->setStreetAddress(isset($customer['address']) ? $customer['address'] : null)
                    ->setCity(isset($customer['city']) ? $customer['city'] : null)
                    ->setZipCode(isset($customer['postcode']) ? $customer['postcode'] : null);

                /* Options */
                $Options
                    ->setTags(empty($this->tags) ? array() : $this->tags)
                    ->setCreatedOn(isset($customer['created_on']) ? $customer['created_on'] : null);

                $ContactsCollection->addItem($Contact);
            }
            return $ContactsCollection;
        } catch (Exception $e) {
            $this->message = $e->getViewMessage();
            $this->status = self::FAILED;
            $this->buildResponse();

        } catch (\Exception $e) {
            $this->message = $e->getMessage();
            $this->status = self::FAILED;
            $this->buildResponse();
        }
        return null;
    }

    /**
     * @param $collection
     * @return EventsCollection|null
     */
    public function prepareEventsToExport($collection)
    {
        try {
            $EventsCollection = new EventsCollection();

            foreach ($collection as $event) {
                if (empty($event['email'])) {
                    continue;
                }

                $Event = new Event();

                $Event
                    ->setEmail(isset($event['email']) ? $event['email'] : null)
                    ->setDate(isset($event['date']) ? $event['date'] : null)
                    ->setDescription(isset($event['description']) ? $event['description'] : null)
                    ->setProducts(isset($event['products']) ? $event['products'] : null)
                    ->setValue(isset($event['value']) ? $event['value'] : null)
                    ->setContactExtEventType(isset($event['contactExtEventType']) ? $event['contactExtEventType'] : self::PURCHASE)
                    ->setExternalId(isset($event['externalId']) ? $event['externalId'] : null)
                    ->setShopDomain(isset($event['shopDomain']) ? $event['shopDomain'] : get_site_url())
                    ->setDetails(array(
                        '1' => isset($event['detail1']) ? $event['detail1'] : null,
                        '2' => isset($event['detail2']) ? $event['detail2'] : null,
                        '3' => isset($event['detail3']) ? $event['detail3'] : null
                    ));

                $EventsCollection->addItem($Event);
            }
            return $EventsCollection;
        } catch (Exception $e) {
            $this->message = $e->getViewMessage();
            $this->status = self::FAILED;
            $this->buildResponse();

        } catch (\Exception $e) {
            $this->message = $e->getMessage();
            $this->status = self::FAILED;
            $this->buildResponse();
        }
        return null;
    }

    /**
     * @param false $count - return count query instead of data query
     * @return string|null
     */
    public function getExportContactsQuery($count = false)
    {
        try {
            $limit = self::PACKAGE_SIZE;
            $offset = $limit * ($this->lastExportedPackage + 1);

            $query = '';
            if ($count) {
                $query .= "SELECT COUNT(*) AS count FROM (";
            }

            $query .= "
            SELECT A.ID as order_id,
                   B.meta_value as first_name,
                   C.meta_value as last_name,
                   D.meta_value as address_1,
                   E.meta_value as address_2,
                   F.meta_value as country,
                   G.meta_value as state,
                   H.meta_value as city,
                   I.meta_value as postcode,
                   J.meta_value as user_id,
                   K.meta_value as email,
                   L.meta_value as phone,
                   A.post_date  as created_on
            
            FROM 
                {$this->db->posts} as A
            LEFT JOIN
                {$this->db->postmeta} B
                    ON A.id = B.post_id AND B.meta_key = '_billing_first_name'
            LEFT JOIN
                {$this->db->postmeta} C
                    ON A.id = C.post_id AND C.meta_key = '_billing_last_name'
            LEFT JOIN
                {$this->db->postmeta} D
                    ON A.id = D.post_id AND D.meta_key = '_billing_address_1'
            LEFT JOIN
                {$this->db->postmeta} E
                    ON A.id = E.post_id AND E.meta_key = '_billing_address_2'
            LEFT JOIN
                {$this->db->postmeta} F
                    ON A.id = F.post_id AND F.meta_key = '_billing_country'
            LEFT JOIN
                {$this->db->postmeta} G
                    ON A.id = G.post_id AND G.meta_key = '_billing_state'
            LEFT JOIN
                {$this->db->postmeta} H
                    ON A.id = H.post_id AND H.meta_key = '_billing_city'
            LEFT JOIN
                {$this->db->postmeta} I
                    ON A.id = I.post_id AND I.meta_key = '_billing_postcode'
            LEFT JOIN 
                {$this->db->postmeta} J
                    ON A.id = J.post_id AND J.meta_key = '_customer_user'
            LEFT JOIN
                {$this->db->postmeta} K
                    ON A.id = K.post_id AND K.meta_key = '_billing_email'
            LEFT JOIN
                {$this->db->postmeta} L
                    ON A.id = L.post_id AND L.meta_key = '_billing_phone'
            
            WHERE
                  A.post_type = 'shop_order'
            AND
                  A.post_date >= '{$this->dateFrom}'
              AND
                  A.post_date <= '{$this->dateTo}'
            ";

            if (!$count) {
                $query .= "        
                LIMIT {$limit}
                    
                OFFSET {$offset}
                ";
            }

            if ($count) {
                $query .= ") AS qwerty";
            }
            return trim(preg_replace('/\s\s+/', ' ', $query));
        } catch (\Exception $e) {
            $this->message = $e->getMessage();
            $this->status = self::FAILED;
            $this->buildResponse();
        }
        return null;
    }

    /**
     * @param false $count - return count instead of data
     * @return array|false|int
     */
    public function getEventsData($count = false)
    {
        try {
            $data = [];
            $page = $this->lastExportedPackage + 2;

            $argGetOrders['date_created'] = strtotime($this->dateFrom);
            $argGetOrders['date_created'] .= '...';
            $argGetOrders['date_created'] .= strtotime($this->dateTo);

            $argGetOrders = [
                'status' => 'wc-completed',
                'limit' => self::PACKAGE_SIZE,
                'page' => $page
            ];

            if($count) {
                $argGetOrders = [
                    'status' => 'wc-completed',
                    'limit' => -1,
                    'page' => ''
                ];
                return count(wc_get_orders($argGetOrders));
            }

            if ($orders = wc_get_orders($argGetOrders)) {
                if (empty($orders)) {
                    return false;
                }

                foreach ($orders as $order) {
                    if ($order->get_items()) {
                        $products = $order->get_items();
                        $prodArr = [
                            'ids' => [],
                            'names' => [],
                            'quantity' => []
                        ];

                        foreach ($products as $product) {

                            $arrayProducts = $product->get_data();
                            $quantity = $arrayProducts['quantity'];

                            if ($quantity > 0) {

                                $prodArr['ids'][] = ($product->get_product_id())
                                    ? $product->get_product_id()
                                    : '';
                                $prodArr['names'][] = ($product->get_name())
                                    ? $product->get_name()
                                    : '';
                                $prodArr['quantity'][] = ($product->get_quantity())
                                    ? $product->get_quantity()
                                    : '';
                            }
                        }

                        if ($quantity > 0) {

                            if ($order->get_billing_email()) {
                                $data[] = [
                                    'email' => $order->get_billing_email(),
                                    'date' => ($order->get_date_created()->getTimestamp())
                                        ? $order->get_date_created()->getTimestamp() * 1000
                                        : '',
                                    'description' => ($order->get_payment_method_title())
                                        ? $order->get_payment_method_title()
                                        : '',
                                    'products' => is_array($prodArr['ids'])
                                        ? implode(',', $prodArr['ids'])
                                        : $prodArr['ids'],
                                    'value' => ($order->get_total())
                                        ? $order->get_total()
                                        : '',
                                    'contactExtEventType' => self::PURCHASE,
                                    'detail1' => is_array($prodArr['names'])
                                        ? implode(',', $prodArr['names'])
                                        : '',
                                    'detail2' => ($order->get_order_key())
                                        ? $order->get_order_key()
                                        : '',
                                    'detail3' => is_array($prodArr['quantity'])
                                        ? implode('/', $prodArr['quantity'])
                                        : $prodArr['quantity'],
                                    'externalId' => ($order->get_id())
                                        ? $order->get_id()
                                        : '',
                                    'shopDomain' => get_site_url()
                                ];
                            }
                        }
                    }
                }
                return $data;
            }
        } catch (Exception $e) {
            $this->message = $e->getViewMessage();
            $this->status = self::FAILED;
            $this->buildResponse();

        } catch (\Exception $e) {
            $this->message = $e->getMessage();
            $this->status = self::FAILED;
            $this->buildResponse();
        }
        return array();
    }
}