<?php

namespace bhr\Admin\Model;

if(!defined('ABSPATH')) exit;

use ErrorException;
use SALESmanago\Exception\Exception;

class Helper
{
    /**
     * @param $array
     * @return bool
     */
    public static function isEmpty($array){
        if(is_array($array)){
            foreach($array as $value){
                if(!empty($value)){
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * @param $input - array or string with commas
     * @param bool $returnString - true to receive string, false for array
     * @param false $removeInnerSpaces - spaces within items will be replaced with underscore
     * @param false $toUpperCase
     * @param string $separator - use separator different than comma
     * @return array|string
     */
    public static function clearCSVInput($input, $returnString = true, $removeInnerSpaces = false, $toUpperCase = false, $separator = ',')
    {
        if(empty($input)) {
            return $returnString ? '' : array();
        }
        if(is_string($input) && !is_array($input)) {
            $inputArray = explode(',', $input);
        } else {
            $inputArray = $input;
        }
        $output = array();
        foreach ($inputArray as $item) {
            if(trim($item)) {
                $outItem = $toUpperCase
                    ? strtoupper(trim($item))
                    : trim($item);
                $outItem = $removeInnerSpaces
                    ? str_replace(' ', '_', $outItem)
                    : $outItem;
                $output[] = $outItem;
            }
        }
        return $returnString
            ? implode($separator, $output)
            : $output;
    }

    public static function filterArray($arr)
    {
        if(!is_array($arr)) {
            throw new Exception("$arr is not an array!");
        }
        $out = array();
        foreach ($arr as $item) {
            $out[] = $item;
        }
        return $out;
    }

    public static function setErrorHandler()
    {
        set_error_handler('bhr\Admin\Model\ErrorToExceptionHandler');
    }
}

/**
 * @param $severity
 * @param $message
 * @param $file
 * @param $line
 * @throws ErrorException
 */
function ErrorToExceptionHandler($severity, $message, $file, $line)
{
    if(!(error_reporting() & $severity)) {
        return;
    }
    throw new ErrorException($message, 0, $severity, $file, $line);
}