<?php

namespace bhr\admin\View;

if(!defined('ABSPATH')) exit;

use bhr\Admin\Entity\MessageEntity;
use bhr\Admin\Model\AdminModel;

use bhr\Admin\Entity\Plugins\Cf7;
use bhr\Admin\Entity\Plugins\Gf;


class SettingsRenderer
{
    private $AdminModel;

    public function __construct(AdminModel $AdminModel)
    {
        $this->AdminModel = $AdminModel;
    }

    /**
     *
     */
    public function getSettingsView()
    {
        try {
            $page = $this->AdminModel->getPage();
            $userLogged = $this->AdminModel->getUserLogged();
        } catch (\Exception $e) {
            MessageEntity::getInstance()
                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 602 : $e->getCode()));
        }

        echo('<div class="wrap" id="salesmanago">');
        echo('
        <a href="https://salesmanago.com/login.htm" target="_blank">
            <img id="salesmanago-logo" src="'.$this->AdminModel->getPluginUrl().'src/Admin/View/img/logo.svg" alt="SALESmanago"/>
        </a>');
        echo(MessageEntity::getInstance()->getMessagesHtml());
        if(empty($page)) {
            echo('</div>');
            return;
        }
        try {
            /* User logged */
            if ($userLogged) {
                include(dirname(__FILE__) . '/partials/navbar.php');

                switch ($page) {
                    case 'salesmanago':
                        try {
                            include(dirname(__FILE__) . '/integration_settings.php');
                        } catch (\Exception $e) {
                            MessageEntity::getInstance()
                                ->setMessagesAfterView(true)
                                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 610 : $e->getCode()));
                        }
                        break;
                    case 'salesmanago-login':
                        try {
                            include(dirname(__FILE__) . '/login_form.php');
                        } catch (\Exception $e) {
                            MessageEntity::getInstance()
                                ->setMessagesAfterView(true)
                                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 110 : $e->getCode()));
                        }
                        break;
                    case 'salesmanago-export':
                        try {
                            $installedDate = $this->AdminModel->getPluginInstalledDate();
                            include(dirname(__FILE__) . '/export.php');
                        } catch (\Exception $e) {
                            MessageEntity::getInstance()
                                ->setMessagesAfterView(true)
                                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 620 : $e->getCode()));
                        }
                        break;
                    case 'salesmanago-plugins':
                        try {
                            include(dirname(__FILE__) . '/plugins.php');
                        } catch (\Exception $e) {
                            MessageEntity::getInstance()
                                ->setMessagesAfterView(true)
                                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 630 : $e->getCode()));
                        }
                        break;
                    case 'salesmanago-plugin-wp':
                        try {
                            $context = 'wp';
                            include(dirname(__FILE__) . '/plugins/plugin_wp.php');
                        } catch (\Exception $e) {
                            MessageEntity::getInstance()
                                ->setMessagesAfterView(true)
                                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 640 : $e->getCode()));
                        }
                        break;
                    case 'salesmanago-plugin-wc':
                        try {
                            $context = 'wc';
                            if (!$this->AdminModel->getInstalledPluginByName('wc')) {
                                echo('<div class="notice notice-warning">' . __('This plugin was not detected.', 'salesmanago') . '</div>');
                            }
                            include(dirname(__FILE__) . '/plugins/plugin_wc.php');
                        } catch (\Exception $e) {
                                MessageEntity::getInstance()
                                    ->setMessagesAfterView(true)
                                    ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 650 : $e->getCode()));
                        }
                        break;
                    case 'salesmanago-plugin-cf7':
                        try {
                            $context = 'cf7';
                            if (!$this->AdminModel->getInstalledPluginByName('cf7')) {
                                echo('<div class="notice notice-warning">' . __('This plugin was not detected.', 'salesmanago') . '</div>');
                            }
                            $availableFormsList = Cf7::listAvailableForms();
                            include(dirname(__FILE__) . '/plugins/plugin_c7.php');
                        } catch (\Exception $e) {
                            MessageEntity::getInstance()
                                ->setMessagesAfterView(true)
                                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 660 : $e->getCode()));
                        }
                        break;
                    case 'salesmanago-plugin-gf':
                        try {
                            $context = 'gf';
                            if (!$this->AdminModel->getInstalledPluginByName('gf')) {
                                echo('<div class="notice notice-warning">' . __('This plugin was not detected.', 'salesmanago') . '</div>');
                            } else {
                                $availableFormsList = Gf::listAvailableForms();
                            }
                            include(dirname(__FILE__) . '/plugins/plugin_gf.php');
                        } catch (\Exception $e) {
                            MessageEntity::getInstance()
                                ->setMessagesAfterView(true)
                                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 670 : $e->getCode()));
                        }
                        break;
                    default:
                        include(dirname(__FILE__) . '/integration_settings.php');
                        break;
                }

                /* User not logged */
            } else {
                if ($page == 'salesmanago') {
                    include(dirname(__FILE__) . '/login_form.php');
                    return;
                }
            }

            /* Always available */
            if ($page == 'salesmanago-go-to-app') {
                include(dirname(__FILE__) . '/go_to_app.php');
                return;
            }
        } catch (\Exception $e) {
            MessageEntity::getInstance()
                ->setMessagesAfterView(true)
                ->addException(new \Exception($e->getMessage(), $e->getCode()==0 ? 602 : $e->getCode()));
        }
        if(MessageEntity::getInstance()->isMessagesAfterView()) {
            echo(MessageEntity::getInstance()->getMessagesHtml());
        }
        //closing of main wrap #salesmanago
        echo('</div>');
    }

    /**
     * @param $value
     * @param $name
     * @param string $context
     * @return bool|string
     */
    public function selected($value, $name, $context = '')
    {
        try {
            switch ($name) {
                case 'cookie-ttl':
                    if (!empty($this->AdminModel->getConfiguration()->getCookieTtl())) {
                        echo ($this->AdminModel->getConfiguration()->getCookieTtl() == $value) ? 'selected' : '';
                    }
                    break;
                case 'language-detection':
                    if (!empty($this->AdminModel->getPlatformSettings()->getLanguageDetection())) {
                        echo ($this->AdminModel->getPlatformSettings()->getLanguageDetection() == $value) ? 'selected' : '';
                    }
                    break;
                case 'disable-monitoring-code':
                    echo ($this->AdminModel->getPlatformSettings()->isDisableMonitoringCode()) ? 'checked' : '';
                    break;
                case 'plugins':
                    echo ($this->AdminModel->getPlatformSettings()->isActive($value)) ? 'checked' : '';
                    break;
                case 'double-opt-in-active':
                    if ($this->AdminModel->getPlatformSettings()->getPluginByName($context)->getDoubleOptIn()->isActive()) {
                        return 'checked';
                    }
                    return '';
                case 'owner':
                    if (!empty($this->AdminModel->getPlatformSettings()->getPluginByName($context)->getOwner())) {
                        return ($this->AdminModel->getPlatformSettings()->getPluginByName($context)->getOwner() == $value)
                            ? 'selected'
                            : '';
                    }
                    break;
                case 'opt-in-input-active':
                    return $this->AdminModel->getPlatformSettings()->getPluginByName($context)->getOptInInput()->getMode() == 'none'
                        ? false
                        : true;
                case 'opt-in-input-mode':
                    $mode = $this->AdminModel->getPlatformSettings()->getPluginByName($context)->getOptInInput()->getMode();
                    if (empty($value)) {
                        return $mode;
                    } else {
                        echo ($value === $mode) ? 'selected' : '';
                    }
                    break;
                case 'purchase-hook':
                    $purchaseHook = $this->AdminModel->getPlatformSettings()->getPluginByName($context)->getPurchaseHook();
                    echo ($value === $purchaseHook) ? 'selected' : '';
                    break;
                case 'prevent-event-duplication':
                    echo ($this->AdminModel->getPlatformSettings()->getPluginByName($context)->isPreventEventDuplication()) ? 'checked' : '';
                    break;
            }
        } catch (\Exception $e) {
                MessageEntity::getInstance()
                    ->setMessagesAfterView(true)
                    ->addException(new \Exception($e->getMessage(), $e->getCode() == 0 ? 602 : $e->getCode()));
            }
        return '';
    }

    /**
     * @param $tab
     */
    public function active($tab)
    {
        echo ($this->AdminModel->getPage() == $tab) ? 'nav-tab-active' : '';
    }

    /**
     * @param $tab
     * @return bool
     */
    public function available($tab)
    {
        return ($this->AdminModel->isTabAvailable($tab));
    }
}
