<?php

namespace bhr\Frontend\Model;

if(!defined('ABSPATH')) exit;

use SALESmanago\Entity\Contact\Contact;
use SALESmanago\Entity\Contact\Address;
use SALESmanago\Entity\Contact\Options;
use SALESmanago\Entity\Contact\Properties;
use SALESmanago\Exception\Exception;

class AbstractContactModel extends AbstractModel
{
    protected $Contact;
    protected $Address;
    protected $Options;
    protected $Properties;

    protected $PlatformSettings;
    protected $PluginSettings;

    public function __construct($PlatformSettings, $PluginSettings) {
        parent::__construct();
        $this->Contact          = new Contact();
        $this->Address          = new Address();
        $this->Options          = new Options();
        $this->Properties       = new Properties();

        $this->PlatformSettings = $PlatformSettings;
        $this->PluginSettings   = $PluginSettings;

        $this->Contact->setAddress($this->Address);
        $this->Contact->setOptions($this->Options);
        $this->Contact->setProperties($this->Properties);
    }

    /**
     * @return Contact
     */
    public function get()
    {
        return $this->Contact;
    }

    /**
     *
     */
    public function setOptInStatuses()
    {
        try {
            if (!isset($this->Contact) || !isset($this->Options)) {
                throw new Exception('Contact or Options were not constructed');
            }

            //check if module newsletter is mapping an opt-in field
            if(!empty($this->PluginSettings->OptInInput->mode)
                && $this->PluginSettings->OptInInput->mode == 'map'
                && !empty($this->PluginSettings->OptInInput->mappedName)
            ) {
                $optInFieldName = $this->PluginSettings->OptInInput->mappedName;
            }

            //Check if submitted form contains checked checkbox named 'sm-optin' or 'sm_newsletter'
            if (
                //Field appended with Opt In Input append (sm-optin):
                (isset($_REQUEST[self::OPT_IN]) && $_REQUEST[self::OPT_IN])

                //Field mapped with mapper
                || (!empty($optInFieldName) && isset($_REQUEST[$optInFieldName]) && $_REQUEST[$optInFieldName])

                //Legacy support for previous input name:
                || (isset($_REQUEST['sm_newsletter']) && $_REQUEST['sm_newsletter'])
            ) {
                $this->Options->setIsSubscribes(true);
                $this->Options->setIsSubscriptionStatusNoChange(false);
                $this->setTagsFromConfig(self::TAGS_NEWSLETTER);
            }
        } catch (Exception $e) {
            $e->getLogMessage();
        }
    }

    /**
     *
     */
    public function setLanguage()
    {
        if(isset($this->PlatformSettings->languageDetection)) {
            if($this->PlatformSettings->languageDetection === 'browser') {
                $lang = substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 2);
            } else {
                $lang = substr(get_user_locale(), 0, 2);
            }
            $this->Options->setLang(strtoupper($lang));
        }
    }
}
