<?php

namespace bhr\Frontend\Model;

if(!defined('ABSPATH')) exit;

use bhr\Frontend\Plugins\Cf7\Cf7Controller;
use bhr\Frontend\Controller\TransferController;
use bhr\Frontend\Model\Settings as SettingsModel;
use bhr\Frontend\Plugins\Gf\GfController;
use bhr\Frontend\Plugins\Wc\WcController;
use bhr\Frontend\Plugins\Wp\WpController;

class HooksModel
{
    private $SettingsModel;

    public function __construct(SettingsModel $SettingsModel)
    {
        $this->SettingsModel = $SettingsModel;

        /* Monitoring code hook */
        if (!isset($this->SettingsModel->getPlatformSettings()->disableMonitoringCode)
        || !$this->SettingsModel->getPlatformSettings()->disableMonitoringCode) {
            add_action("wp_print_footer_scripts", array($this, "addMonitoringCode"));
        }

        /* WordPress hooks */
        if ($this->SettingsModel->getPlatformSettings()->PluginWp->active) {
            add_action('user_register', array($this, 'initWp'));
            add_action('wp_login', array($this, 'initWp'));
            if($this->SettingsModel->getPlatformSettings()->PluginWp->OptInInput->mode === 'append') {
                add_action('register_form',array($this, 'appendCheckbox'),14);
                add_action('woocommerce_register_form', array($this, 'appendCheckbox'),14);
            }
        }

        /* WooCommerce hooks */
        if ($this->SettingsModel->getPlatformSettings()->PluginWc->active) {
            /* Order Hooks */
            add_action('woocommerce_order_status_cancelled', array($this, 'initWc'), 10, 1);
            add_action('woocommerce_order_status_refunded', array($this, 'initWc'), 10, 1);
            // NOTE: purchase hook is set in PurchaseModel

            /* User Hooks */
            add_action('woocommerce_checkout_update_user_meta', array($this, 'initWc'));
            add_action('woocommerce_customer_save_address', array($this, 'initWc'));
            add_action('profile_update', array($this, 'initWc'));
            add_action('user_register', array($this, 'initWc'));
            add_action('wp_login', array($this, 'initWc'));

            /* Cart Hooks */
            add_action('woocommerce_add_to_cart', array($this, 'initWc'));
            add_action('woocommerce_update_cart_action_cart_updated', array($this, 'initWc'));
            add_action('woocommerce_remove_cart_item', array($this, 'initWc'));

            /* Purchase Hook */
            $purchaseHook = $this->SettingsModel->getPlatformSettings()->PluginWc->purchaseHook;
            if(!empty($purchaseHook)) {
                add_action($purchaseHook, array($this, 'initWc'));
            } else {
                add_action('woocommerce_order_status_changed', array($this, 'initWc'));
            }
            /* Opt-in input Hooks */
            if($this->SettingsModel->getPlatformSettings()->PluginWc->OptInInput->mode === 'append') {
                add_action('woocommerce_register_form', array($this, 'appendCheckbox'),14);
                //add_action('woocommerce_after_checkout_registration_form', array($this, 'appendCheckbox'), 15);
            } elseif($this->SettingsModel->getPlatformSettings()->PluginWc->OptInInput->mode === 'appendEverywhere') {
                add_action('woocommerce_register_form', array($this, 'appendCheckbox'),14);
                //add_action('woocommerce_after_checkout_registration_form', array($this, 'appendCheckbox'), 15);
                add_action('woocommerce_review_order_before_submit', array($this, 'appendCheckbox'),14);
            }
        }

        /* Contact Form 7 hook */
        if ($this->SettingsModel->getPlatformSettings()->PluginCf7->active) {
            add_action('wpcf7_mail_sent', array($this, 'initCf7'), 5);
        }

        /* Gravity Forms hook */
        if ($this->SettingsModel->getPlatformSettings()->PluginGf->active) {
            add_action('gform_after_submission', array($this, 'initGf'), 10, 2);
        }

    }

    /**
     *
     */
    public function addMonitoringCode()
    {
        $clientId = $this->SettingsModel->getConfiguration()->getClientId();
        $endpoint = $this->SettingsModel->getConfiguration()->getEndpoint();

        if(empty($clientId) || empty($endpoint)) {
            return;
        }

        $code = "<script>var _smid ='{$clientId}';
         (function(w, r, a, sm, s ) {
         w['SalesmanagoObject'] = r;
         w[r] = w[r] || function () {( w[r].q = w[r].q || [] ).push(arguments)};
         sm = document.createElement('script');
         sm.type = 'text/javascript'; sm.async = true; sm.src = a;
         s = document.getElementsByTagName('script')[0];
         s.parentNode.insertBefore(sm, s);
         })(window, 'sm', '{$endpoint}/static/sm.js');</script>";

        print trim(preg_replace('/\s+/', ' ', $code));
    }

    /**
     * @param $data
     */
    public function initWp($data)
    {
        $TransferController = new TransferController(
            $this->SettingsModel->getConfiguration(),
            $this->SettingsModel->getPlatformSettings()->PluginWp
        );
        if(!$WpController = new WpController($this->SettingsModel->getPlatformSettings(), $TransferController)) {
            return false;
        }

        $currentHook = current_filter();
        switch ($currentHook) {
            case 'user_register':
                $WpController->registerUser($data);
                break;
            case 'wp_login':
                $WpController->loginUser($data);
                break;
        }
        return true;
    }

    /**
     * @param $data
     */
    public function initWc($data)
    {
        $TransferController = new TransferController(
            $this->SettingsModel->getConfiguration(),
            $this->SettingsModel->getPlatformSettings()->PluginWc
        );
        if(!$WcController = new WcController($this->SettingsModel->getPlatformSettings(), $TransferController)) {
            return false;
        }

        $currentHook = current_filter();
        switch ($currentHook) {
            case 'woocommerce_order_status_cancelled':
            case 'woocommerce_order_status_refunded':
                $WcController->orderStatusChanged($data);
                break;
            case 'woocommerce_checkout_update_user_meta':
            case 'woocommerce_customer_save_address':
            case 'profile_update':
                $WcController->createUser($data);
                break;
            case 'user_register':
                $WcController->registerUser($data);
                break;
            case 'wp_login':
                $WcController->loginUser($data);
                break;
            case 'woocommerce_add_to_cart':
            case 'woocommerce_update_cart_action_cart_updated':
            case 'woocommerce_remove_cart_item':
                $WcController->addToCart();
                break;
            case 'woocommerce_order_status_changed':
                $WcController->purchase($data);
                break;
            default:
                $WcController->purchase($data);
                break;
        }
        return true;
    }

    /**
     * @param $data
     */
    public function initCf7($data)
    {
        $TransferController = new TransferController(
            $this->SettingsModel->getConfiguration(),
            $this->SettingsModel->getPlatformSettings()->PluginCf7
        );
        if(!$Cf7Controller = new Cf7Controller($this->SettingsModel->getPlatformSettings(), $TransferController)) {
            return false;
        }
        $Cf7Controller->execute($data);
        return true;
    }

    /**
     * @param $formValues
     * @param $formSetup
     */
    public function initGf($formValues, $formSetup)
    {
        $TransferController = new TransferController(
            $this->SettingsModel->getConfiguration(),
            $this->SettingsModel->getPlatformSettings()->PluginGf
        );
        if(!$GfController = new GfController($this->SettingsModel->getPlatformSettings(), $TransferController)) {
            return false;
        }
        $GfController->execute($formValues, $formSetup);
        return true;
    }

    /**
     *
     */
    public function appendCheckbox()
    {
        $cssClass  = 'sm-opt-in-input';
        $inputName = 'sm-optin';

         $label = ($this->SettingsModel->getPlatformSettings()->PluginWc->active)
             ? $this->SettingsModel->getPlatformSettings()->PluginWc->OptInInput->label
             : $this->SettingsModel->getPlatformSettings()->PluginWp->OptInInput->label;

        load_plugin_textdomain( 'salesmanago', false, 'salesmanago/languages' );

        //Get translation for input label. If there is no translation or using default language, use label declared in admin settings.
        $displayLabel = (__('!optInInputLabel', 'salesmanago') === '!optInInputLabel')
            ? $label
            : __('!optInInputLabel', 'salesmanago');

        echo('<p class="woocommerce-FormRow form-row ' . $cssClass . '">
            <label class="woocommerce-form__label woocommerce-form__label-for-checkbox inline ' . $cssClass . '" style="margin-left: 0">
                <input name="' . $inputName. '" class="woocommerce-form__input woocommerce-form__input-checkbox ' . $cssClass . '" type="checkbox">
                <span class="' . $cssClass . '">' . $displayLabel . '</span>
            </label>
        </p>');
    }
}
