<?php

namespace bhr\Frontend\Plugins\Cf7;

if(!defined('ABSPATH')) exit;

use bhr\Frontend\Model\AbstractContactModel;
use SALESmanago\Entity\Contact\Contact;
use SALESmanago\Exception\Exception;

class Cf7ContactModel extends AbstractContactModel
{
    private   $currentFormConfig;

    public function __construct($PlatformSettings)
    {
        //do not continue without settings
        if(empty($PlatformSettings) || empty($PlatformSettings->PluginCf7)) {
            return false;
        }
        //create an Abstract Contact
        parent::__construct($PlatformSettings, $PlatformSettings->PluginCf7);
        return true;
    }

    /**
     * @param $id
     * @return bool
     */
    public function setCurrentFormConfig($id)
    {
        //Get config (tags, owner) for submitted form
        $id = intval($id);
        if(isset($this->PluginSettings->forms->$id)) {
            $this->currentFormConfig = $this->PluginSettings->forms->$id;
            return true;
        }
        return false;
    }

    /**
     * @return mixed
     */
    public function getCurrentFormConfig()
    {
        return $this->currentFormConfig;
    }

    /**
     * @return Contact|null
     * @throws Exception
     */
    public function parseContact($formData)
    {
        try {
            //No email no fun
            if (!isset($formData['sm-email']) || empty($formData['sm-email'])) {
                return null;
            }

            /* Contact */
            $this->Contact
                ->setEmail($formData['sm-email'])
                ->setName(isset($formData['sm-name'])             ? $formData['sm-name'] : '')
                ->setPhone(isset($formData['sm-phone'])           ? $formData['sm-phone'] : '')
                ->setFax(isset($formData['sm-fax'])               ? $formData['sm-fax'] : '')
                ->setCompany(isset($formData['sm-company'])       ? $formData['sm-company'] : '');

            /* Address */
            $this->Address
                ->setCity(isset($formData['sm-city'])             ? $formData['sm-city'] : '')
                ->setCountry(isset($formData['sm-country'])       ? $formData['sm-country'] : '')
                ->setStreetAddress(isset($formData['sm-address']) ? $formData['sm-address'] : '')
                ->setZipCode(isset($formData['sm-postcode'])      ? $formData['sm-postcode'] : '');

            /* Options */
            $this->setLanguage();
            $this->Options
                ->setTags(
                    isset($this->currentFormConfig->tags)
                        ? $this->currentFormConfig->tags
                        : ''
                )->setRemoveTags(
                    isset($this->currentFormConfig->tagsToRemove)
                        ? $this->currentFormConfig->tagsToRemove
                        : ''
                );

            if (isset($formData['sm-optin'])
                && ((is_array($formData['sm-optin'])
                && implode($formData['sm-optin']) != '')
                || boolval($formData['sm-optin']))
            ) {
                $this->Options->setIsSubscribes(true); //Set flag for subscription
                $this->Options->setIsSubscriptionStatusNoChange(false); //Set flag - opt-in status has changed
            }

            /* Custom properties */
            $this->Properties->setItems($this->getProperties($formData));

            return $this->Contact;
        } catch (\Exception $e) {
            throw new Exception($e->getMessage());
        }
    }


    /**
     * @param $formData
     * @return array
     */
    private function getProperties($formData)
    {
        $properties = array();
        if(isset($this->PluginSettings->properties)) {
            foreach ($this->PluginSettings->properties as $propertyName) {
                $customInput = isset($formData[$propertyName]) ? $formData[$propertyName] : '';
                $customInput = is_array($customInput) ? implode(',', $customInput) : $customInput;
                $customInput = (strlen($customInput) > 255) ? substr($customInput, 0, 255) : $customInput;

                if ($propertyName != '' && $customInput != '') {
                    $properties[$propertyName] = $customInput;
                }
            }
        }
        return $properties;
    }
}