<?php

namespace bhr\Frontend\Plugins\Wc;

if(!defined('ABSPATH')) exit;

use bhr\Frontend\Model\AbstractContactModel;
use SALESmanago\Entity\Contact\Contact;
use SALESmanago\Exception\Exception;

class WcContactModel extends AbstractContactModel
{
    const
        NICK_N = 'nickname',
        F_NAME = 'first_name',
        L_NAME = 'last_name',
        DESC   = 'description',
        LOCALE = 'locale',

        B_EMAIL     = 'billing_email',
        B_F_NAME    = 'billing_first_name',
        B_L_NAME    = 'billing_last_name',
        B_CITY      = 'billing_city',
        B_COMPANY   = 'billing_company',
        B_PHONE     = 'billing_phone',
        B_POSTCODE  = 'billing_postcode',
        B_ADDRESS_1 = 'billing_address_1',
        B_ADDRESS_2 = 'billing_address_2',
        B_COUNTRY   = 'billing_country',

        P_NO_ACC_EMAIL     = '_billing_email',
        P_NO_ACC_COMPANY   = '_billing_company',
        P_NO_ACC_F_NAME    = '_billing_first_name',
        P_NO_ACC_L_NAME    = '_billing_last_name',
        P_NO_ACC_PHONE     = '_billing_phone',
        P_NO_ACC_ADDRESS_1 = '_billing_address_1',
        P_NO_ACC_ADDRESS_2 = '_billing_address_2',
        P_NO_ACC_POSTCODE  = '_billing_postcode',
        P_NO_ACC_CITY      = '_billing_city',
        P_NO_ACC_COUNTRY   = '_billing_country',

        SINGLE_VALUE       = true,
        ID                 = 'id',
        LOGIN              = 'login';


    public  $user;      //holds userId or user login
    public  $userType;  //login or register

    public function __construct($PlatformSettings)
    {
        //do not continue without settings
        if(empty($PlatformSettings) || empty($PlatformSettings->PluginWc)) {
            return false;
        }
        //create an Abstract Contact
        parent::__construct($PlatformSettings, $PlatformSettings->PluginWc);
        return true;
    }

    /**
     * @param $user
     * @param string $userType
     * @return false|Contact|null
     */
    public function parseContact($user, $userType = 'id')
    {
        $this->user     = $user;
        $this->userType = $userType;
        if(!isset($this->user) || empty($this->user)) {
            return null;
        }

        $contactData = '';
        if ($this->userType === self::ID) {
            $contactData = \WP_User::get_data_by('id', $this->user);
        } elseif ($this->userType === self::LOGIN) {
            $contactData = get_user_by('login', $this->user);
        }

        if (empty($contactData)) {
            return false;
        }

        /* email */
        if (!isset($contactData->user_email) || empty($contactData->user_email)) {
            return false;
        }
        $this->Contact->setEmail($contactData->user_email);

        /* name
        Try to get name from Billing Address, Account Details, or Request
         */
        $name = trim(get_user_meta($contactData->ID, self::F_NAME, true) . ' '
            . get_user_meta($contactData->ID, self::L_NAME, true));
        if(empty($name)) {
            $name = trim(get_user_meta($contactData->ID, self::B_F_NAME, true) . ' '
                . get_user_meta($contactData->ID, self::B_L_NAME, true));
        }
        if (empty($name) && isset($_REQUEST['action']) && $_REQUEST['action'] == 'save_account_details') {
            $name = trim($_REQUEST['account_first_name'] . ' '
                . $_REQUEST['account_last_name']);
        }
        $this->Contact->setName($name);

        /* phone */
        $phone = get_user_meta($contactData->ID, self::B_PHONE, self::SINGLE_VALUE);
        $this->Contact->setPhone($phone);

        /* company */
        $company = get_user_meta($contactData->ID, self::B_COMPANY, self::SINGLE_VALUE);
        $this->Contact->setCompany($company);

        /* options */
        $this->setLanguage();
        $this->setOptInStatuses();

        return $this->Contact;
    }

    /**
     * @param $orderId
     * @return Contact|null
     */
    public function parseCustomer($orderId)
    {
        /* email */
        $email = get_post_meta($orderId, self::P_NO_ACC_EMAIL, self::SINGLE_VALUE);
        if(empty($email)) {
            return null;
        }
        $this->Contact->setEmail($email);

        /* name */
        $name = trim(get_post_meta($orderId, self::P_NO_ACC_F_NAME, self::SINGLE_VALUE) . ' ' .
            get_post_meta($orderId, self::P_NO_ACC_L_NAME, self::SINGLE_VALUE));
        $this->Contact->setName(!empty($name) ? $name : '');

        /* phone */
        $phone = get_post_meta($orderId, self::P_NO_ACC_PHONE, self::SINGLE_VALUE);
        $this->Contact->setPhone(!empty($phone) ? $phone : '');

        /* company */
        $company = get_post_meta($orderId, self::P_NO_ACC_COMPANY, self::SINGLE_VALUE);
        $this->Contact->setCompany(!empty($company) ? $company : '');

        /* streetAddress */
        $streetAddress = trim(get_post_meta($orderId, self::P_NO_ACC_ADDRESS_1, self::SINGLE_VALUE) . ' ' .
            get_post_meta($orderId, self::P_NO_ACC_ADDRESS_2, self::SINGLE_VALUE));
        $this->Address->setStreetAddress(!empty($streetAddress) ? $streetAddress : '');

        /* zipCode */
        $zipCode = get_post_meta($orderId, self::P_NO_ACC_POSTCODE, self::SINGLE_VALUE);
        $this->Address->setZipCode(!empty($zipCode) ? $zipCode : '');

        /* city */
        $city = get_post_meta($orderId, self::P_NO_ACC_CITY, self::SINGLE_VALUE);
        $this->Address->setCity(!empty($city) ? $city : '');

        /* country */
        $country = get_post_meta($orderId, self::P_NO_ACC_COUNTRY, self::SINGLE_VALUE);
        $this->Address->setCountry(!empty($country) ? $country : '');

        $this->Contact->setAddress($this->Address);

        /* options */
        $this->setLanguage();
        $this->setOptInStatuses();

        return $this->Contact;
    }

    /**
     * @return array|null
     */
    public static function getSmClient()
    {
        $smclient = isset($_COOKIE['smclient']) ? $_COOKIE['smclient'] : null;
        if(!$smclient) {
            $smclient = isset($_SESSION['smclient']) ? $_SESSION['smclient'] : null;
        }
        return $smclient;
    }

    /**
     * @return mixed|null
     */
    public function getClientEmail()
    {
        try {
            if (!empty($this->Contact->getEmail())) {
                return $this->Contact->getEmail();
            }
        } catch (\Exception $e) {
            //silence is gold
        }

        if (get_current_user_id()) {
            $currentUser = wp_get_current_user()->data;
            return !empty($currentUser->user_email) ? $currentUser->user_email : null;
        }
        return null;
    }

    /**
     * @return Contact|null
     */
    public function parseCustomerFromPost()
    {
        /* email */
        $email = empty($_REQUEST[self::B_EMAIL]) ? '' : $_REQUEST[self::B_EMAIL];
        if(empty($email)) {
            return null;
        }
        $this->Contact->setEmail($email);

        /* name */
        $name = trim($_REQUEST[self::B_F_NAME] . ' ' .
            $_REQUEST[self::B_L_NAME]);
        $this->Contact->setName(!empty($name) ? $name : '');

        /* phone */
        $phone = $_REQUEST[self::B_PHONE];
        $this->Contact->setPhone(!empty($phone) ? $phone : '');

        /* company */
        $company = $_REQUEST[self::B_COMPANY];
        $this->Contact->setCompany(!empty($company) ? $company : '');


        /* streetAddress */
        $streetAddress = trim($_REQUEST[self::B_ADDRESS_1] . ' ' .
            $_REQUEST[self::B_ADDRESS_2]);
        $this->Address->setStreetAddress(!empty($streetAddress) ? $streetAddress : '');

        /* zipCode */
        $zipCode = $_REQUEST[self::B_POSTCODE];
        $this->Address->setZipCode(!empty($zipCode) ? $zipCode : '');

        /* city */
        $city = $_REQUEST[self::B_CITY];
        $this->Address->setCity(!empty($city) ? $city : '');

        /* country */
        $country = $_REQUEST[self::B_COUNTRY];
        $this->Address->setCountry(!empty($country) ? $country : '');

        $this->Contact->setAddress($this->Address);

        /* options */
        $this->setLanguage();
        $this->setOptInStatuses();

        return $this->Contact;
    }

    /**
     * @param $tagsType
     * @return Contact
     */
    public function setTagsFromConfig($tagsType)
    {
        $tags = isset($this->PluginSettings->tags->$tagsType)
            ? $this->PluginSettings->tags->$tagsType
            : null;
        if(!empty($tags)) {
            $tags = explode(',', $tags);
            $this->Options->appendTags($tags);
        }
        return $this->Contact;
    }
}
