<?php

namespace bhr\Frontend\Plugins\Wc;

if(!defined('ABSPATH')) exit;

use bhr\Frontend\Model\Helper;
use bhr\Frontend\Model\AbstractModel;
use SALESmanago\Entity\Event\Event;
use SALESmanago\Exception\Exception;

class WcEventModel extends AbstractModel
{
    const
        COOKIES_EXT_EVENT       = "smevent",
        EVENT_TYPE_CART         = "CART",
        EVENT_TYPE_PURCHASE     = "PURCHASE",
        EVENT_TYPE_CANCELLATION = "CANCELLATION",
        EVENT_TYPE_RETURN       = "RETURN",
        EVENT_TYPE_DEFAULT      = "OTHER";

    protected $Event;

    public function __construct()
    {
        parent::__construct();
        $this->Event = new Event();
    }

    /**
     * @param $client
     * @param $eventType
     * @param $products
     * @param null $smevent
     * @return Event|null
     * @throws Exception
     */
    public function bindEvent($client, $eventType, $products, $smevent = null)
    {
        try {
            $this->Event
                ->setContactExtEventType($eventType)
                ->setProducts(isset($products["products"])?$products["products"]:'')
                ->setDescription(isset($products["description"])?$products["description"]:'')
                ->setValue(isset($products["value"])?$products["value"]:'')
                ->setLocation(self::getLocation())
                ->setDetails(array(
                        '1' => isset($products["detail1"]) ? $products["detail1"] : '',
                        '2' => isset($products["detail2"]) ? $products["detail2"] : '',
                        '3' => isset($products["detail3"]) ? $products["detail3"] : '',
                        '4' => isset($products["detail4"]) ? $products["detail4"] : '',
                        '5' => isset($products["detail5"]) ? $products["detail5"] : ''
                        )
                    )
                ->setEventId($smevent)
                ->setDate(time());
            if (isset($client[self::SM_CLIENT]) && !empty($client[self::SM_CLIENT])) {
                $this->Event->setContactId($client[self::SM_CLIENT]);
            }
            elseif (isset($client[self::EMAIL]) && !empty($client[self::EMAIL])) {
                $this->Event->setEmail($client[self::EMAIL]);
            }
            elseif (isset($client) && !empty($client)) {
                $this->Event->setEmail($client);
            }
            else {
                return null;
            }
            return $this->Event;
            } catch (\Exception $e) {
                throw new Exception($e->getMessage());
            }
    }

    /**
     * @return Event
     */
    public function get()
    {
        return $this->Event;
    }

    /**
     * @param $orderId
     * @return array
     */
    public function getProductsFromOrder($orderId)
    {
        /* Products */
        $order = wc_get_order($orderId);
        $ids   = array();
        $names = array();
        $quantities = array();

        foreach ($order->get_items() as $item_id => $item ) {
            $ids[]        = $item['product_id'];
            $names[]      = $item['name'];
            $quantities[] = $item->get_quantity();
        }

        /* Shipping */
        $shippingMethodId   = WC()->session->get( 'chosen_shipping_methods' );
        $shippingMethodNames = array();

        foreach(WC()->cart->get_shipping_packages() as $packageId => $package) {
            if(WC()->session->__isset('shipping_for_package_'.$packageId)) {
                foreach(WC()->session->get('shipping_for_package_'.$packageId)['rates'] as $shippingRate ) {
                    if(in_array($shippingRate->get_id(), $shippingMethodId))  {
                        $shippingMethodNames[] = $shippingRate->get_label();
                    }
                }
            }
        }
        $shippingMethodName = implode(',', $shippingMethodNames);

        $product = array(
            'description' => $order->get_payment_method(),
            'products'    => implode(',', $ids),
            'value'       => $order->get_total(),
            'detail1'     => implode(',', $names),
            'detail2'     => $order->get_order_key(),
            'detail3'     => implode('/', $quantities),
            'detail4'     => $order->get_customer_note(),
            'detail5'     => $shippingMethodName,
            'externalId'  => $order->get_id()
        );

        return $product;
    }

    /**
     * @return array
     */
    public function getProductsFromCart()
    {
        global $woocommerce;

        $ids       = [];
        $names     = [];
        $quantities = [];
        $cartTotalPrice   = 0;

        foreach ($woocommerce->cart->get_cart() as $item) {
            $id = $item['data']->get_id();
            $WcProduct = wc_get_product($id);

            $ids[]             = $WcProduct->get_id();
            $names[]           = $WcProduct->get_name();
            $quantities[]      = $item['quantity'];
            $cartTotalPrice    += (float) ($item['quantity'] * $WcProduct->get_price());
        }

        $CartRecoveryModel = new WcCartRecoveryModel();
        try {
            $cartRecoveryUrl = $CartRecoveryModel->getCartRecoveryUrl();
        } catch (\Exception $e) {
            $cartRecoveryUrl = '';
        }

        $products = array(
            'description' => $cartRecoveryUrl,
            'products'    => implode(',', $ids),
            'value'       => $cartTotalPrice,
            'detail1'     => implode(',', $names),
            'detail3'     => implode('/', $quantities),
        );
        return $products;
    }

    /**
     * @return mixed|null
     */
    public function getSmEvent()
    {
        $smevent = isset($_COOKIE['smevent']) ? $_COOKIE['smevent'] : null;
        if (!$smevent) {
            $smevent = isset($_SESSION['smevent']) ? $_SESSION['smevent'] : null;
        }
        return empty($smevent) ? null : $smevent;
    }

    /**
     * @return string
     */
    private static function getLocation()
    {
        $shopLocation = (get_permalink(wc_get_page_id('shop')) == -1)
            ? get_home_url()
            : get_permalink( wc_get_page_id( 'shop' ));
        return empty($shopLocation)
            ? md5($_SERVER['SERVER_NAME'])
            : md5($shopLocation);
    }
}